@@ -10,6 +10,7 @@ from dateutil import parser
 from functools import reduce, partial
 import dateutil as du
 from dateutil import tz
+from dateutil.zoneinfo import tzfile
 from pytz import country_timezones
 import pytz
 
@@ -41,15 +42,20 @@ def tzones():
     tzs  = {key : tz.gettz(value) for key, value in tzs.items()}
     t = datetime.datetime.now()
     tzs.update({v.tzname(t): v for v in tzs.values() if v is not None})
+    tzs['DE']  = tzs['berlin'] # in pytz both 'Europe/Berlin', 'Europe/Busingen' are listed by Germany only has one timezone
     tzs['WET'] = tzs['western european'] = tzs['MA'] # Malta
     tzs['EET'] = tzs['eastern european'] = tzs['GR']
     tzs['CET'] = tzs['cental european'] = tzs['AT'] # austria
-    tzs['EST'] = tzs['eastern standard'] = tzs['new york']
-    tzs['CST'] = tzs['central standard'] = tzs['chicago']
+    tzs['EST'] = tzs['ET'] = tzs['eastern standard'] = tzs['new york']
+    tzs['CST'] = tzs['CT'] = tzs['central standard'] = tzs['chicago']
+    tzs['UTC'] = tzs['utc'] = pytz.timezone('UTC')
     tzs['greenwich mean time'] = tzs['GMT']
     return tzs
 
 def as_tz(tzinfo):
+    """
+    can use any country iso
+    """
     if is_str(tzinfo):
         zones = tzones()
         if tzinfo in zones:
@@ -63,57 +69,67 @@ def as_tz(tzinfo):
     else:
         return tzinfo
 
+
+def is_tz(tzinfo):
+    return isinstance(tzinfo, (tzfile, pytz.tzfile.DstTzInfo))
+
+
+
+def tz_replace(t, tzinfo = None):
+    """
+    this REPLACES the tzinfo while not changing the value of the underlying data
+    """
+    tz = as_tz(tzinfo)
+    if isinstance(t, (pd.DataFrame, pd.Series)):
+        return t.tz_localize(None).tz_localize(tz)
+    elif is_date(t):
+        return t.replace(tzinfo = tz)
+    elif isinstance(t, list):
+        return type(t)([tz_replace(i, tz) for i in t])
+    elif isinstance(t, dict):
+        return type(t)({key : tz_replace(value, tz) for key, value in t.items()})
+    else:
+        return dt(t, tzinfo = tzinfo)
+
+        
 def tz_convert(t, tzinfo = None):
     """
-    if tzinfo is None: drops all timezone data
-    if tzinfo is not None: converts/localize the existing date to 
     
-
     Parameters
     ----------
-    t : TYPE
-        DESCRIPTION.
-    tzinfo : TYPE, optional
-        DESCRIPTION. The default is None.
+    t : date, timeseries
+        if t is a timeseries, will use panda tz_localize or tz_convert
+    tzinfo : None/str/timezone, optional
+        tzinfo that as_tz() can work with. The default is None: remove all tz info
 
     Returns
     -------
-    TYPE
-        DESCRIPTION.
+    t converted to timezone
+    If tzinfo is None: drops all timezone data
+    If tzinfo is not None: converts/localize the existing date to timezone, depending if original data has tz info
+
+    Example
+    --------
 
     """
     if tzinfo is None:
-        ## we remove tzinfo
-        if isinstance(t, (pd.DataFrame, pd.Series)):
-            if t.index.tz is None:
-                return t
-            else:
-                res = t.copy()
-                res.index = [date.replace(tzinfo = None) for date in res.index]
-                return res
-        elif is_date(t):
-            return t if t.tzinfo is None else t.replace(tzinfo = None)
-        elif isinstance(t, list):
-            return type(t)([tz_convert(i, tzinfo) for i in t])
-        elif isinstance(t, dict):
-            return type(t)({key : tz_convert(value, tzinfo) for key, value in t.items()})
-        else:
-            return dt(t, tzinfo = tzinfo)
+        return tz_replace(t)
     if isinstance(t, list):
         return type(t)([tz_convert(i, tzinfo) for i in t])
     elif isinstance(t, dict):
         return type(t)({key : tz_convert(value, tzinfo) for key, value in t.items()})
     if isinstance(t, (pd.DataFrame, pd.Series)):
+        tzinfo = as_tz(tzinfo)
         res = t.copy()
         if t.index.tz is None:
-            res.index = res.index.tz_localize(as_tz(tzinfo))
+            res.index = res.index.tz_localize(tzinfo)
         else:
-            res.index = res.index.tz_convert(as_tz(tzinfo))
+            res.index = res.index.tz_convert(tzinfo)
         return res
     elif is_date(t):
         return t.astimezone(as_tz(tzinfo))
     else:
-        return dt(t, tzinfo)
+        return dt(t, tzinfo = tzinfo)
 
 def today(date = None):
     now = date or datetime.datetime.now()
@@ -133,12 +149,8 @@ def month(m):
     :Parameters:
     ----------------
     m : str/int
-        DESCRIPTION.    
+        month as integer or Futures code.    
 
-    Raises
-    ------
-    ValueError
-        DESCRIPTION.
 
     :Returns:
     -------
@@ -292,7 +304,7 @@ def uk2dt(t, tzinfo = None):
             raise ValueError('date %s is not in UK date format'%t)
     elif yyyymm.search(t) is not None or yyyymmm.search(t) is not None:
         res = datetime.datetime(res.year, res.month, 1)
-    return tz_convert(res, tzinfo)
+    return tz_replace(res, tzinfo)
 
 
 def us2dt(t, tzinfo = None):
@@ -307,7 +319,7 @@ def us2dt(t, tzinfo = None):
         raise ValueError('the date is not in US format')
     if yyyymm.search(t) is not None or yyyymmm.search(t) is not None:
         res = datetime.datetime(res.year, res.month, 1)
-    return tz_convert(res, tzinfo)
+    return tz_replace(res, tzinfo)
 
 def none2dt(none = datetime.datetime.now):
     if callable(none):
@@ -351,6 +363,8 @@ def dt_bump(t, *bumps, aggregate = 'last'):
     >>> t = pd.Series(range(10), drange(9))
     >>> assert len(dt_bump(t, '1b')) < len(t) ## we have gone over the weekends and weekends are bumped forward together
     >>> assert (dt_bump(t, '1b') - dt_bump(t, '1b', aggregate = 'first')).max() == 2 ## 2 days for weekend 
+    
+    t = dt(0); bumps = ('1h',)
     """
     bumps = as_list(bumps)
     if is_ts(t):
@@ -404,9 +418,11 @@ def dt_bump(t, *bumps, aggregate = 'last'):
                     t += DAY * d
             if len(bump):
                 try:
-                    t = tz_convert(t, bump)
+                    return tz_convert(t, bump)
                 except Exception:
                     raise ValueError('%s is not a period I know...'%bump)
+        elif is_tz(bump):
+            t = tz_convert(t, bump)
         else:
             t = t + bump
     return t
@@ -441,6 +457,45 @@ def dt(*args, dialect = 'uk', none = datetime.datetime.now, tzinfo = None):
     >>> assert dt(2000,'h', 1) == datetime.datetime(2000,3,1) # future codes
 
 
+    Timezone handling.
+    ------------------
+    NOTE: dt() and dt_bump() differ in the handling.
+
+    if tzinfo is provided, 
+        - dt will tz_replace, WITHOUT CHANGING DATE.
+        - the only exception is dt(tzinfo) which will give you the time right now, in the local tzinfo
+    
+    if tzinfo is None, 
+        - dt will NOT touch the timezone information and will not remove it. i.e. dt(t) will leave t's timezone intact
+        - if you want to remove tzinfo, use tz_replace(date)
+        
+    
+    Example: construction of time now in different timezones
+    --------
+    >>> assert dt(tzinfo = 'EST').hour == dt(tzinfo = 'CST').hour % 24 + 1
+
+    Example: dt does not drop input timezone if tzinfo
+    --------
+    >>> tz = as_tz('EST')
+    >>> t = dt(tz)
+    >>> assert dt(t).tzinfo == tz
+    >>> assert dt(t, '1h').tzinfo == tz
+    >>> assert dt(t, 1).tzinfo == tz
+
+    >>> t = dt(pd.Series(1, drange(10)), tzinfo = tz)
+    >>> assert dt(t, '1h').index[0].tzinfo == tz
+    
+    Example: dt REPLACES rather than bumps if a tzinfo is provided
+    --------
+    >>> jerusalem = as_tz('jerusalem')
+    >>> london = as_tz('london')
+    >>> j = dt(pd.Series(1, drange(10)), tzinfo = jerusalem)
+    >>> assert set(j.index.hour) == {0}
+    >>> l = dt(j, tzinfo = london)
+    >>> assert set(l.index.hour) == {0} ## the hours have not been bumped
+    >>> l = dt_bump(j, london)
+    >>> assert set(l.index.hour) == {22} ## the hours bumped two hours back
+
     :Example: date as offset from today
     -----------------------------------
     >>> today = dt(0); 
@@ -478,20 +533,26 @@ def dt(*args, dialect = 'uk', none = datetime.datetime.now, tzinfo = None):
     
     """
     args = as_list(args)
+    if tzinfo is None and len(args) and is_tz(args[-1]):
+        tzinfo = args[-1]; args = args[:-1]    
     if len(args) == 0:
         res = none() if callable(none) else none
         return tz_convert(res, tzinfo)
     t = args[0]
     if isinstance(t, np.datetime64):
-        t = tz_convert(np2dt(t), tzinfo)
+        t = np2dt(t)
+        if tzinfo:
+            t = tz_replace(t, tzinfo)
     elif isinstance(t, NaTType):
         return NaT
     elif isinstance(t, datetime.date) and not isinstance(t, datetime.datetime):
         t = datetime.datetime(t.year, t.month, t.day, tzinfo = as_tz(tzinfo))
     elif is_ts(t):
-        return dt_bump(t, *args[1:])
+        res = dt_bump(t, *args[1:])
+        return res if tzinfo is None else tz_replace(res, tzinfo)
     if isinstance(t, datetime.datetime):
-        return reduce(dt_bump, args[1:], tz_convert(t, tzinfo))
+        res = reduce(dt_bump, args[1:], t)
+        return res if tzinfo is None else tz_replace(res, tzinfo)
     if len(args) == 1:
         if t is None:
             return tz_convert(none(), tzinfo) if callable(none) else none
@@ -499,11 +560,12 @@ def dt(*args, dialect = 'uk', none = datetime.datetime.now, tzinfo = None):
             if is_nan(t):
                 return tz_convert(none(), tzinfo) if callable(none) else none
             else:                
-                return tz_convert(num2dt(t), tzinfo)
+                return tz_replace(num2dt(t), tzinfo)
         elif is_bump(t):
             return dt_bump(dt(0, tzinfo = tzinfo), t)
         elif is_str(t):
-            return uk2dt(t, tzinfo = tzinfo) if dialect == 'uk' else us2dt(t, tzinfo = tzinfo)
+            res = uk2dt(t) if dialect == 'uk' else us2dt(t)
+            return res if tzinfo is None else tz_replace(res, tzinfo)    
                 # return int2dt(int(t)) + datetime.timedelta(float(t) % 1)
         else:
             raise ValueError('date format unrecognised %s'%t)
@@ -513,12 +575,14 @@ def dt(*args, dialect = 'uk', none = datetime.datetime.now, tzinfo = None):
     y,m,d = args[:3]
     t = _ymd(y,m,d)
     if len(args) == 4 and is_str(args[3]):
-        return tz_convert(nth_weekday_of_month(*args), tzinfo)
+        return tz_replace(nth_weekday_of_month(*args), tzinfo)
     if len(args) > 3:
         args = [int(a) for a in args[3:]] + [0,0,0]
-        return tz_convert(t + datetime.timedelta(hours = args[0], minutes = args[1], seconds = args[2]), tzinfo)
+        res = t + datetime.timedelta(hours = args[0], minutes = args[1], seconds = args[2])
+        return res if tzinfo is None else tz_replace(res, tzinfo)
     else:
-        return tz_convert(t, tzinfo)
+        res = t
+        return res if tzinfo is None else tz_replace(res, tzinfo)
 
 
 def nth_weekday_of_month(y, m, n, w):
